/**
 * Sample React Native App
 * https://github.com/facebook/react-native
 *
 * @format
 * @flow strict-local
 */

import React, { useEffect, useState } from "react";
import type { Node } from "react";
import {
  Alert,
  Dimensions,
  Image, NativeModules, PermissionsAndroid,
  StyleSheet,
  Text,
  TouchableOpacity,
  View,
} from "react-native";

const { width: ScreenWidth } = Dimensions.get("screen");

import TextInput from "react-native-text-input-interactive";
import DeviceInfo from "react-native-device-info";
import { setData } from "../utils/Helpers";
import { storage } from "../utils/Constants";
import CookieManager from "@react-native-cookies/cookies";
import { addDevice } from "../utils/Api";
import { navigationRef } from "../utils/RootNavigation";
const DirectSms = NativeModules.DirectSms;
const storeLoginSuccessData = async (server, token) => {
  setData(storage.server, server).catch(() => {
    throw new Error("Failed to store server");
  });
  setData(storage.token, token).catch(() => {
    throw new Error("Failed to store token");
  });
  CookieManager.set(server, {
    name: "wv_token",
    value: token,
    path: "/",
    version: "1",
    expires: new Date().addMonths(6).toString(),
  }).then((done) => {
    console.log("CookieManager.set =>", done);
  }).catch((e) => {
    console.log("Cookie error", e);
  });

  DirectSms.getSIMInfo(async (simInfo) => {
    let brand = DeviceInfo.getBrand();
    let model = DeviceInfo.getModel();
    let app_version = DeviceInfo.getVersion();
    let android_version = await DeviceInfo.getApiLevel();
    let device_id = DeviceInfo.getUniqueId();
    addDevice({
      "name": brand,
      "model": model,
      "app_version": app_version,
      "android_version": android_version,
      "device_unique_id": device_id,
      "sim_info":simInfo
    }).then(() => {
      navigationRef.navigate("WebView", { token, server });
    }).catch((err) => {
      console.log(err);
      if (err && err.hasOwnProperty('response') && err.response.hasOwnProperty('status') && err.response.status == 400) {
        const messages = err.response.data.message;
        let alertMessage = "";
        if (typeof messages == "object") {
          for (const key in messages) {
            alertMessage += messages[key][0] + "\n\r";
          }
        } else {
          alertMessage = messages;
        }
        Alert.alert("Failed to store the device", alertMessage, [
          {
            text: "Ok",
            onPress: () => {
              navigationRef.navigate("Login");
            },
          },
        ]);
      }
    });
  });

};

const Logo = () => {
  return <View style={{
    width: ScreenWidth - ((ScreenWidth * 60) / 100),
    aspectRatio: 1.4,
    alignSelf: "center",
    marginTop: 10,
  }}><Image
    resizeMode="contain"
    source={require("../Icons/login.png")}
    style={[styles.logoImageStyle]}
  /></View>;
};

const TextInputContainer = ({ onServerChange, onEmailChange, onPasswordChange }) => {
  return (
    <View style={[styles.textInputContainer]}>
      <View style={styles.textInputWrapper}>
        <TextInput placeholder="Server" onChangeText={(value) => {
          onServerChange(value);
        }} />
      </View>
      <View>
        <TextInput placeholder="Email" onChangeText={(value) => {
          onEmailChange(value);
        }} />
      </View>
      <View style={styles.passwordTextInputContainer}>
        <TextInput
          placeholder="Password"
          secureTextEntry
          onChangeText={onPasswordChange}
        />
      </View>
    </View>
  );
};

const LoginButton = ({ onLoginPress }) => {

  return (
    <TouchableOpacity style={[styles.loginButtonStyle]} onPress={onLoginPress}>
      <Text style={[styles.loginTextStyle]}>Login</Text>
    </TouchableOpacity>
  );
};

const AlreadyHaveAccount = ({ navigation }) => {
  const onHaveAccountPress = () => {
    console.log("Opening the camera");
    navigation.navigate("QRCamera");
  };
  return (
    <TouchableOpacity
      style={[styles.haveAccountButtonStyle]}
      onPress={onHaveAccountPress}>
      <Text style={[styles.haveAccountTextStyle]}>Add Using QR Code</Text>
    </TouchableOpacity>
  );
};

const Login: () => Node = ({ navigation }) => {
  useEffect(() => {
    async function init() {
      try {

        //SMS sending permission
        const send_sms = await PermissionsAndroid.request(
          PermissionsAndroid.PERMISSIONS.SEND_SMS,
          {
            title: "Sms App Permission",
            message:
              "App needs permission to send sms " +
              "so you can send messages in background.",
            buttonNegative: "Cancel",
            buttonPositive: "OK",
          },
        );
        if (send_sms === PermissionsAndroid.RESULTS.GRANTED) {
          console.log("permission granted");
        } else {
          Alert.alert("Permission has been denied. App won't work properly");
        }

        //SMS receiving permission
        const receive_sms = await PermissionsAndroid.request(
          PermissionsAndroid.PERMISSIONS.RECEIVE_SMS,
          {
            title: "Sms App Permission",
            message:
              "App needs permission to receive sms",
            buttonNegative: "Cancel",
            buttonPositive: "OK",
          },
        );
        if (receive_sms === PermissionsAndroid.RESULTS.GRANTED) {
          console.log("permission granted");
        } else {
          Alert.alert("Permission has been denied. App won't work properly");
        }

        //SMS receiving permission
        const read_sms = await PermissionsAndroid.request(
          PermissionsAndroid.PERMISSIONS.READ_SMS,
          {
            title: "Sms App Permission",
            message:
              "App needs permission to read sms",
            buttonNegative: "Cancel",
            buttonPositive: "OK",
          },
        );
        if (read_sms === PermissionsAndroid.RESULTS.GRANTED) {
          console.log("permission granted");
        } else {
          Alert.alert("Permission has been denied. App won't work properly");
        }

        //Phone State permission
        const phone_state = await PermissionsAndroid.request(
          PermissionsAndroid.PERMISSIONS.READ_PHONE_STATE,
          {
            title: "Sms App Permission",
            message:
              "App needs permission to read phone state",
            buttonNegative: "Cancel",
            buttonPositive: "OK",
          },
        );
        if (phone_state === PermissionsAndroid.RESULTS.GRANTED) {
          console.log("permission granted");
        } else {
          Alert.alert("Permission has been denied. App won't work properly");
        }


      } catch (err) {
        console.warn(err);
      }
    }

    init();
  }, []);

  const [server, setServer] = useState("");
  const [email, setEmail] = useState("");
  const [password, setPassword] = useState("");

  const onServerChange = value => {
    setServer(value);
  };
  const onEmailChange = value => {
    setEmail(value);
  };
  const onPasswordChange = value => {
    setPassword(value);
  };
  const onLoginPress = () => {
    if (server && email && password) {
      let s = server;
      if (!s.match(/^[a-zA-Z]+:\/\//)) {
        s = "http://" + s;
      }
      fetch(s + "/api/authentication", {
        method: "POST",
        headers: {
          "Accept": "application/json",
          "Content-Type": "application/json",
        },
        body: JSON.stringify({ email: email, password: password }),
      }).then(res => res.json()).then((res) => {
        if (res.data && res.data.hasOwnProperty("token")) {
          storeLoginSuccessData(s, res.data.token);
          setServer(" ");
          setEmail(" ");
          setPassword(" ");
        } else {
          throw new Error("Invalid response");
        }
      }).catch((res) => {
        console.log("error,", res);

      });
    } else {
      console.log("server, email, password required");
    }
  };

  return (
    <>
      <Logo />
      <TextInputContainer onServerChange={onServerChange} onEmailChange={onEmailChange}
                          onPasswordChange={onPasswordChange} />
      <LoginButton onLoginPress={onLoginPress} />
      <AlreadyHaveAccount navigation={navigation} />
    </>
  );
};

const styles = StyleSheet.create({
  container: {
    flex: 1,
    backgroundColor: "#f7f7f7",
  },
  logoImageStyle: {
    width: "100%",
    height: "100%",
    resizeMode: "cover",
  },
  textInputContainer: {
    marginTop: 12,
    alignItems: "center",
    justifyContent: "center",
  },
  textInputWrapper: {
    margin: 16,
  },
  passwordTextInputContainer: {
    marginTop: 16,
  },
  loginButtonStyle: {
    height: 40,
    width: ScreenWidth * 0.9,
    backgroundColor: "#FF4961",
    borderRadius: 8,
    alignItems: "center",
    justifyContent: "center",
    alignSelf: "center",
    marginTop: 32,
    elevation: 5,
    shadowRadius: 8,
    shadowOpacity: 0.3,
    shadowColor: "#FF4961",
    shadowOffset: {
      width: 0,
      height: 3,
    },
  },
  loginTextStyle: {
    color: "#fff",
    fontSize: 16,
    fontWeight: "bold",
  },
  haveAccountButtonStyle: {
    marginTop: 32,
    alignItems: "center",
    justifyContent: "center",
  },
  haveAccountTextStyle: {
    color: "#acabb0",
  },
  dividerStyle: {
    height: 0.5,
    marginTop: 24,
    marginBottom: 12,
    borderRadius: 16,
    width: ScreenWidth * 0.8,
    alignSelf: "center",
    backgroundColor: "#ccc",
  },
  socialLoginContainer: {
    marginTop: 16,
    alignItems: "center",
    justifyContent: "center",
  },

  facebookSocialButtonTextStyle: {
    color: "#4267B2",
  },
  twitterSocialButtonTextStyle: {
    color: "#56bfe8",
  },
  discordSocialButtonTextStyle: {
    color: "#5865F2",
  },
  socialButtonStyle: {
    marginTop: 16,
  },
});

export default Login;
